﻿Type=Class
Version=2.02
@EndOfDesignText@
'Class module
Sub Class_Globals
	Private Window As Panel
	Private Icon As ImageView
	Private IconText As Label

	Private wParent As Panel
	Private wModule As Object
	Private wPressedBackground As Object

	Private wStartWidth, wStartHeight As Int
	Private wStartX, wStartY As Int

	Private wDockingAreas As List
	Private wDocked As Boolean
	Private OnEnterDockAreaWinSub, OnLeaveDockAreaWinSub, OnDockWinSub, OnUndockWinSub
	Private Anim As Animation
	Private wLastRect As Rect
	Private C As Canvas
End Sub

'Initializes the movable icon
'Parent = activity or panel holding the icon
'DockingAreas = list of Rect defining the docking areas
'Module = Me
Public Sub Initialize(Parent As Panel, Left As Int, Top As Int, Width As Int, Height As Int, Image As Bitmap, Text As String, DockingAreas As List, Module As Object)
	Window.Initialize("Window")
	Parent.AddView(Window, Left, Top, Width, Height)
	wParent = Parent
	wStartWidth = Width
	wStartHeight = Height
	Dim r As Reflector
	r.Target = Window
	r.SetOnTouchListener("Window_onTouch")

	Icon.Initialize("")
	Icon.Bitmap = Image
	Icon.Gravity = Gravity.FILL
	Window.AddView(Icon, 0, 0, Width, Width)

	IconText.Initialize("")
	Dim cd As ColorDrawable
	cd.Initialize(Colors.ARGB(220, 30, 30, 30), 10dip)
	IconText.Background = cd
	IconText.Gravity = Gravity.CENTER_HORIZONTAL + Gravity.CENTER_VERTICAL
	IconText.Text = Text
	IconText.TextColor = Colors.White
	IconText.TextSize = 13
	Dim lblWidth As Int
	C.Initialize(Window)
	lblWidth = Min(Width, C.MeasureStringWidth(IconText.Text, IconText.Typeface, IconText.TextSize) + 8dip)
	Window.AddView(IconText, (Width - lblWidth) / 2, Width + 5dip, lblWidth, Height - Width - 10dip)

	wDockingAreas = DockingAreas
	wDocked = False

	wPressedBackground = LoadDrawable("pressed_application_background_static")
	wModule = Module
End Sub

'Gets a drawable from the Android system resources
Public Sub LoadDrawable(Name As String) As Object
	Dim r As Reflector
	r.Target = r.GetContext
	r.Target = r.RunMethod("getResources")
	r.Target = r.RunMethod("getSystem")
	Dim ID_Drawable As Int
	ID_Drawable = r.RunMethod4("getIdentifier", Array As Object(Name, "drawable", "android"), _
	                                            Array As String("java.lang.String", "java.lang.String", "java.lang.String"))
	r.Target = r.GetContext
	r.Target = r.RunMethod("getResources")
	Return r.RunMethod2("getDrawable", ID_Drawable, "java.lang.int")
End Sub

#Region Left, Top, Width, Height
'Gets the window left value
Public Sub getLeft
	Return Window.Left
End Sub

'Gets the window top value
Public Sub getTop
	Return Window.Top
End Sub

'Gets the window width
Public Sub getWidth
	Return Window.Width
End Sub

'Gets the window height
Public Sub getHeight
	Return Window.Height
End Sub
#End Region

Private Sub Window_LongClick
	' Brings the window to the front
	Window.BringToFront

	' Removes the "pressed" background
	Dim cd As ColorDrawable
	cd.Initialize(Colors.Transparent, 0)
	Window.Background = cd

	' Undocks the window
	wDocked = False
	If SubExists(wModule, OnUndockWinSub) Then CallSub2(wModule, OnUndockWinSub, Me)

	' Enlarges the window
	Enlarge
End Sub

Private Sub Window_onTouch(ViewTag As Object, Action As Int, X As Float, Y As Float, MotionEvent As Object) As Boolean
	If Action = 0 Then ' DOWN
		wStartX = X
		wStartY = Y
		Window.Background = wPressedBackground
	End If
	If wDocked Then
		If Action = 1 Then
			Dim cd As ColorDrawable
			cd.Initialize(Colors.Transparent, 0)
			Window.Background = cd
		End If
		Return False
	End If
	If Action = 2 Then ' MOVE
		MoveWin(getLeft + X - wStartX, getTop + Y - wStartY)
		CheckDockingAreas(False, 0)
	Else If Action = 1 Then ' UP
		CheckDockingAreas(True, 100)
	End If
	Return True
End Sub

Private Sub MoveWin(NewLeft As Int, NewTop As Int)
	'The icon cannot be fully moved outside of its parent
	Window.Left = Min(Max(-getWidth / 2, NewLeft), wParent.Width - (getWidth / 2))
	Window.Top = Min(Max(-getHeight / 2, NewTop), wParent.Height - (getHeight / 2))
End Sub

Private Sub CheckDockingAreas(PerformAction As Boolean, AnimDuration As Int)
	Dim wRect As Rect
	Dim Surface, BiggestSurface As Long
	Dim SelectedArea As Int
	SelectedArea = -1
	For i = 0 To wDockingAreas.Size - 1
		wRect = wDockingAreas.Get(i)
		If (getLeft <= wRect.Right AND getLeft + getWidth >= wRect.Left) AND _
   		(getTop <= wRect.Bottom AND getTop + getHeight >= wRect.Top) Then
			' The icon is over a docking area
			' We calculate the covered area
			Dim SurfaceW, SurfaceH As Int
			If getLeft > wRect.Left Then
				SurfaceW = Min(wRect.Right - getLeft, getWidth)
			Else
				SurfaceW = getLeft + getWidth - wRect.Left
			End If
			If getTop > wRect.Top Then
				SurfaceH = Min(wRect.Bottom - getTop, getHeight)
			Else
				SurfaceH = getTop + getHeight - wRect.Top
			End If
			Surface = SurfaceW * SurfaceH
			If Surface > BiggestSurface Then
				' We select the docking area with the most covered surface
				SelectedArea = i
				BiggestSurface = Surface
			End If
		End If
	Next

	If SelectedArea = -1 Then
		' The icon isn't over a docking area
		' We select the last available docking area
		SelectedArea = wDockingAreas.Size - 1
	End If

	If wLastRect.IsInitialized Then
		If wDockingAreas.Get(SelectedArea) <> wLastRect Then
			' The docking area has changed
			If SubExists(wModule, OnLeaveDockAreaWinSub) Then CallSub3(wModule, OnLeaveDockAreaWinSub, Me, wLastRect)
		End If
	End If
	wLastRect = wDockingAreas.Get(SelectedArea)
	If PerformAction Then
		Anim.InitializeTranslate("Anim", 0, 0, wLastRect.Left - getLeft, wLastRect.Top - getTop)
		Anim.Duration = AnimDuration
		Anim.RepeatCount = 0
		Anim.Start(Window)
	Else
		If SubExists(wModule, OnEnterDockAreaWinSub) Then CallSub3(wModule, OnEnterDockAreaWinSub, Me, wLastRect)
	End If
End Sub

Private Sub Anim_AnimationEnd
	MoveWin(wLastRect.Left, wLastRect.Top)
	RestoreNormalSize
	wDocked = True
	If SubExists(wModule, OnDockWinSub) Then CallSub3(wModule, OnDockWinSub, Me, wLastRect)
	Anim = Null
End Sub

'Docks the icon in the nearest docking area
Public Sub ForceDock
	CheckDockingAreas(True, 0)
End Sub

Private Sub Enlarge
	' Enlarges the window to ease its handling
	Dim bd As BitmapDrawable
	bd.Initialize(ViewToBmp(Window))
	Window.Background = bd

	Icon.Visible = False
	IconText.Visible = False

	Dim Factor As Float: Factor = 1.3
	Window.Width = Round(getWidth * Factor)
	Window.Height = Round(getHeight * Factor)

	Window.Left = getLeft - ((getWidth - wStartWidth) / 2)
	wStartX = wStartX + ((getWidth - wStartWidth) / 2)
	Window.Top = getTop - ((getHeight - wStartHeight) / 2)
	wStartY = wStartY + ((getHeight - wStartHeight) / 2)
End Sub

Sub ViewToBmp(MyView As View) As Bitmap
	' Returns a screenshot of the given view
	Dim bmp As Bitmap
	bmp.InitializeMutable(MyView.Width, MyView.Height)
	C.Initialize2(bmp)

	Dim r1 As Reflector
	r1.Target = C
	r1.Target = r1.GetField("canvas")

	Dim r2 As Reflector
	Dim args(1) As Object
	Dim types(1) As String
	args(0) = r1.Target
	types(0) = "android.graphics.Canvas"
	r2.Target = MyView
	r2.RunMethod4("draw", args, types) ' "Draw" renders a view and its children to the given Canvas

	Return C.Bitmap
End Sub

Private Sub RestoreNormalSize
	' Restores the normal size
	Window.Width = wStartWidth
	Window.Height = wStartHeight
	Window.Color = Colors.Transparent

	Icon.Visible = True
	IconText.Visible = True

	Anim.InitializeScaleCenter("", 1, 1, 0.8, 0.8, Window)
	Anim.Duration = 70
	Anim.RepeatMode = Anim.REPEAT_REVERSE
	Anim.RepeatCount = 1
	Anim.Start(Window)
End Sub

#Region SetListeners
'Adds a listener for the events OnEnterDockArea (fired when entering a docking area)
'Your listener must be declared like this: myListener(Icon As ClsMovableIcon, DockRect As Rect)
Public Sub SetOnEnterDockAreaListener(SubName As String)
	OnEnterDockAreaWinSub = SubName
End Sub

'Adds a listener for the events OnLeaveDockArea (fired when leaving a docking area)
'Your listener must be declared like this: myListener(Icon As ClsMovableIcon, DockRect As Rect)
Public Sub SetOnLeaveDockAreaListener(SubName As String)
	OnLeaveDockAreaWinSub = SubName
End Sub

'Adds a listener for the events OnDock (fired after docking)
'Your listener must be declared like this: myListener(Icon As ClsMovableIcon, DockRect As Rect)
Public Sub SetOnDockListener(SubName As String)
	OnDockWinSub = SubName
End Sub

'Adds a listener for the events OnUndock (fired after undocking)
'Your listener must be declared like this: myListener(Icon As ClsMovableIcon)
Public Sub SetOnUndockListener(SubName As String)
	OnUndockWinSub = SubName
End Sub
#End Region

'Removes the icon from the memory
Public Sub Destroy
	Dim v As View
	For i = Window.NumberOfViews - 1 To 0 Step -1
		v = Window.GetView(i)
		v.RemoveView
	Next
	Window.RemoveView
	Window = Null
End Sub
